import {Component, EventEmitter, Inject, Input, Output} from '@angular/core';
import AuthApiService, {OAuthProviderData} from "@api/v1/authApiService";
import {OAuthProvider} from "@model/oAuthProvider";
import {ToastrService} from "ngx-toastr";
import {
  MAT_DIALOG_DATA, MatDialog,
  MatDialogActions,
  MatDialogContent,
  MatDialogRef,
  MatDialogTitle
} from "@angular/material/dialog";
import {MatButton} from "@angular/material/button";
import {DataSpinnerComponent} from "@component/common/data-spinner/data-spinner.component";

interface AvailableOAuthProviders extends OAuthProviderData {
  disabled: boolean;
  active: boolean;
}

@Component({
  selector: 'app-delete-confirm-dialog',
  template: `
    <h1 mat-dialog-title>Удалить провайдера?</h1>
    <mat-dialog-content>
      <p>Вы уверены, что хотите удалить провайдера {{ data.provider.name }}?</p>
    </mat-dialog-content>
    <mat-dialog-actions style="display: flex; justify-content: flex-end;">
      <button mat-button (click)="onCancel()">Отмена</button>
      <button mat-raised-button color="warn" (click)="onConfirm()">Удалить</button>
    </mat-dialog-actions>
  `,
  imports: [
    MatDialogTitle,
    MatDialogContent,
    MatDialogActions,
    MatButton
  ]
})
export class DeleteConfirmDialog {

  constructor(
    public dialogRef: MatDialogRef<DeleteConfirmDialog>,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {
  }

  onConfirm(): void {
    this.dialogRef.close(true);
  }

  onCancel(): void {
    this.dialogRef.close(false);
  }
}

@Component({
  selector: 'OAuthProviders',
  imports: [
    DataSpinnerComponent
  ],
  templateUrl: './OAuthProviders.html',
  styleUrl: './OAuthProviders.css',
  providers: [AuthApiService]
})
export class OAuthProviders {
  protected providers: AvailableOAuthProviders[] = [];
  protected _activeProvidersId: OAuthProvider[] = [];
  protected _activeProviders: string[] = [];

  @Input() message: string = 'Вы можете войти в аккаунт через';

  @Input() set activeProviders(data: string[]) {
    this._activeProviders = data;
    this.updateDisabledProviders();
  }

  @Input() set activeProvidersId(data: OAuthProvider[]) {
    this._activeProvidersId = data;
    this.updateDisabledProviders();
  }

  @Input() canUnlink: boolean = false;

  @Output() public oAuthUpdateProviders = new EventEmitter();

  constructor(authApi: AuthApiService, private notify: ToastrService, private dialog: MatDialog) {
    authApi.availableProviders().subscribe(providers => this.updateDisabledProviders(providers));
  }

  private updateDisabledProviders(data: OAuthProviderData[] | null = null) {
    this.providers = (data ?? this.providers).map(provider => {
      return {
        ...provider,
        disabled: this._activeProvidersId.includes(provider.provider) || this._activeProviders.includes(provider.providerName),
        active: false
      };
    });
  }

  protected openOAuth(provider: AvailableOAuthProviders) {
    console.log(provider.redirect);
    const oauthWindow = window.open(
      provider.redirect,
      '_blank',
    );

    if (!oauthWindow) {
      this.notify.error('Не удалось открыть OAuth окно');
      return;
    }

    provider.active = true;

    const checkInterval = setInterval(() => {
      if (oauthWindow.closed) {
        clearInterval(checkInterval);
        this.oAuthUpdateProviders.emit();
        provider.active = false;
      }
    }, 1500);
  }

  protected confirmDelete(provider: AvailableOAuthProviders) {
    const dialogRef = this.dialog.open(DeleteConfirmDialog, {data: {provider}});

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.deleteProvider(provider);
      }
    });
  }

  protected deleteProvider(provider: AvailableOAuthProviders) {
    // todo: remove provider
  }
}
